@@ -6,7 +6,7 @@
 --------------------------------------------------------------------------------
 gSOAP XML Web services tools
 Copyright (C) 2001-2012, Robert van Engelen, Genivia Inc. All Rights Reserved.
-This software is released under one of the following two licenses:
+This software is released under one of the following licenses:
 GPL or Genivia's license for commercial use.
 --------------------------------------------------------------------------------
 GPL license.
@@ -37,7 +37,11 @@ A commercial use license is available from Genivia, Inc., contact@genivia.com
 
 #include "stdsoap2.h"
 
-#define WSDL2H_VERSION "2.8.12"
+#ifdef WITH_OPENSSL
+#include "httpda.h"
+#endif
+
+#define WSDL2H_VERSION "2.8.17r"
 
 #ifdef WIN32
 # pragma warning(disable : 4996)
@@ -72,9 +76,9 @@ typedef pair<const char*, const char*> Pair;
 struct ltpair
 { bool operator()(Pair s1, Pair s2) const
   { int cmp = strcmp(s1.first, s2.first);
-    if (cmp)
-      return cmp < 0;
-    return strcmp(s1.second, s2.second) < 0;
+    if (cmp == 0)
+      cmp = strcmp(s1.second, s2.second);
+    return cmp < 0;
   }
 };
 
@@ -100,6 +104,7 @@ extern int _flag,
 	   mflag,
 	   pflag,
 	   Pflag,
+	   Rflag,
 	   sflag,
 	   uflag,
 	   vflag,
@@ -116,7 +121,7 @@ extern SetOfString exturis;
 #define MAXINFILES (1000)
 
 extern int infiles;
-extern char *infile[MAXINFILES], *outfile, *proxy_host, *proxy_userid, *proxy_passwd;
+extern char *infile[MAXINFILES], *outfile, *proxy_host, *proxy_userid, *proxy_passwd, *auth_userid, *auth_passwd;
 extern const char *mapfile, *import_path, *cwd_path, *cppnamespace;
 
 extern int proxy_port;
