@@ -6,7 +6,7 @@
 --------------------------------------------------------------------------------
 gSOAP XML Web services tools
 Copyright (C) 2000-2012, Robert van Engelen, Genivia Inc. All Rights Reserved.
-This software is released under one of the following two licenses:
+This software is released under one of the following licenses:
 GPL or Genivia's license for commercial use.
 --------------------------------------------------------------------------------
 GPL license.
@@ -80,6 +80,9 @@ int show_ignore(struct soap*, const char*);
 
 wsdl__definitions::wsdl__definitions()
 { soap = soap_new1(SOAP_XML_TREE | SOAP_C_UTFSTRING);
+#ifdef HTTPDA_H
+  soap_register_plugin(soap, http_da);
+#endif
 #ifdef WITH_OPENSSL
   soap_ssl_client_context(soap, SOAP_SSL_NO_AUTHENTICATION, NULL, NULL, NULL, NULL, NULL);
 #endif
@@ -188,7 +191,7 @@ int wsdl__definitions::read(const char *cwd, const char *loc)
         *s = '\0';
       strcat(location, "/");
       strcat(location, loc);
-      fprintf(stderr, "\nConnecting to '%s' to retrieve relative '%s' WSDL/XSD...\n", location, loc);
+      fprintf(stderr, "\nConnecting to '%s' to retrieve relative path '%s' WSDL/XSD...\n", location, loc);
       if (soap_connect_command(soap, SOAP_GET, location, NULL))
       { fprintf(stderr, "Connection failed\n");
         exit(1);
@@ -248,7 +251,8 @@ int wsdl__definitions::read(const char *cwd, const char *loc)
       if (soap->error)
       { fprintf(stderr, "\nAn error occurred while parsing WSDL or XSD from '%s'\n", loc?loc:"");
         soap_print_fault(soap, stderr);
-        soap_print_fault_location(soap, stderr);
+        if (soap->error < 200)
+          soap_print_fault_location(soap, stderr);
         exit(1);
       }
       name = NULL;
@@ -271,10 +275,33 @@ int wsdl__definitions::read(const char *cwd, const char *loc)
       redirs--;
       return r;
     }
+    else if (soap->error == 401)
+    { int r = SOAP_ERR;
+      fprintf(stderr, "Authenticating to '%s' realm '%s'...\n", loc, soap->authrealm);
+      if (auth_userid && auth_passwd && redirs++ < 1)
+      { 
+#ifdef HTTPDA_H
+        struct http_da_info info;
+        http_da_save(soap, &info, soap->authrealm, auth_userid, auth_passwd);
+#else
+        soap->userid = auth_userid;
+        soap->passwd = auth_passwd;
+#endif
+        r = read(cwd, loc);
+#ifdef HTTPDA_H
+        http_da_release(soap, &info);
+#endif
+        redirs--;
+      }
+      else
+        fprintf(stderr, "Authentication failed, use option -r:uid:pwd and (re)build with OpenSSL to enable digest authentication\n");
+      return r;
+    }
     else
     { fprintf(stderr, "\nAn error occurred while parsing WSDL from '%s'\n", loc?loc:"");
       soap_print_fault(soap, stderr);
-      soap_print_fault_location(soap, stderr);
+      if (soap->error < 200)
+        soap_print_fault_location(soap, stderr);
       exit(1);
     }
   }
@@ -413,7 +440,8 @@ int wsdl__definitions::error()
 
 void wsdl__definitions::print_fault()
 { soap_print_fault(soap, stderr);
-  soap_print_fault_location(soap, stderr);
+  if (soap->error < 200)
+    soap_print_fault_location(soap, stderr);
 }
 
 void wsdl__definitions::builtinType(const char *type)
@@ -1202,8 +1230,10 @@ xs__complexType *wsdl__part::complexTypePtr() const
 int wsdl__types::preprocess(wsdl__definitions& definitions)
 { if (vflag)
     cerr << "Preprocessing wsdl types" << endl;
-  targetNamespace = definitions.targetNamespace;
-  xs__schema_.push_back(this);
+  if (!empty()) // WSDL 2.0 <types>
+  { targetNamespace = definitions.targetNamespace;
+    xs__schema_.push_back(this);
+  }
   // set the location of each schema in <types> to the WSDL's location
   for (vector<xs__schema*>::iterator schema0 = xs__schema_.begin(); schema0 != xs__schema_.end(); ++schema0)
   { if (!(*schema0)->sourceLocation())
@@ -1213,7 +1243,7 @@ again:
   // link imported schemas, need to repeat when <types> is extended with new imported schema (from inside another schema, etc.)
   for (vector<xs__schema*>::iterator schema1 = xs__schema_.begin(); schema1 != xs__schema_.end(); ++schema1)
   { for (vector<xs__import>::iterator import = (*schema1)->import.begin(); import != (*schema1)->import.end(); ++import)
-    { if ((*import).namespace_ && !(*import).schemaPtr())
+    { if ((*import).namespace_ && !(*import).schemaPtr() && strcmp((*import).namespace_, targetNamespace))
       { for (vector<xs__schema*>::const_iterator schema2 = xs__schema_.begin(); schema2 != xs__schema_.end(); ++schema2)
         { if (schema2 != schema1 && (*schema2)->targetNamespace && !strcmp((*import).namespace_, (*schema2)->targetNamespace))
           { (*import).schemaPtr(*schema2);
@@ -1234,7 +1264,7 @@ again:
       if (!found && (*import).namespace_)
       { if ((*import).schemaPtr())
           found = true;
-        else
+        else if (strcmp((*import).namespace_, targetNamespace))
         { for (vector<xs__schema*>::const_iterator schema3 = xs__schema_.begin(); schema3 != xs__schema_.end(); ++schema3)
           { if (schema3 != schema2 && (*schema3)->targetNamespace && !strcmp((*import).namespace_, (*schema3)->targetNamespace))
             { found = true;
@@ -1276,13 +1306,15 @@ again:
           else if ((*import).namespace_ && strcmp(importschema->targetNamespace, (*import).namespace_))
             cerr << "Schema import namespace '" << ((*import).namespace_?(*import).namespace_:"") << "' does not correspond to imported namespace '" << importschema->targetNamespace << "'" << endl;
         }
-        for (vector<xs__schema*>::const_iterator schema3 = xs__schema_.begin(); schema3 != xs__schema_.end(); ++schema3)
-        { if (schema3 != schema2 && (*schema3)->targetNamespace && !strcmp((*import).namespace_, (*schema3)->targetNamespace))
-          { found = true;
-            (*import).schemaPtr(*schema3);
-            break;
+	if (strcmp((*import).namespace_, targetNamespace))
+        { for (vector<xs__schema*>::const_iterator schema3 = xs__schema_.begin(); schema3 != xs__schema_.end(); ++schema3)
+          { if (schema3 != schema2 && (*schema3)->targetNamespace && !strcmp((*import).namespace_, (*schema3)->targetNamespace))
+            { found = true;
+              (*import).schemaPtr(*schema3);
+              break;
+            }
           }
-        }
+	}
         if (!found)
         { (*import).schemaPtr(importschema);
           xs__schema_.push_back(importschema);
